#include <iostream>

#include <gtk/gtk.h>
#include "imagesource/imagesource.h"
#include "support/patheditor.h"
#include "support/pathsupport.h"
#include "support/generaldialogs.h"
#include "profilemanager/profileselector.h"
#include "profilemanager/intentselector.h"
#include "simplecombo.h"

#include "util.h"

#include "it_gui.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)

#include "dialogs.h"

using namespace std;

#define RESPONSE_SAVE 1


struct prefsdialogdata
{
	IT_GUI *gui;
	GtkWidget *rgbprof;
	GtkWidget *cmykprof;
	GtkWidget *monprof;
	GtkWidget *outputprof;
	GtkWidget *rgbactive;
	GtkWidget *cmykactive;
	GtkWidget *monactive;
	GtkWidget *outputactive;
	GtkWidget *patheditor;
};


static void checkbox_changed(GtkWidget *wid,gpointer *ud)
{
	prefsdialogdata *ob=(prefsdialogdata *)ud;

	int pa=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ob->outputactive));
	int ra=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ob->rgbactive));
	int ca=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ob->cmykactive));
	int ma=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ob->monactive));

	gtk_widget_set_sensitive(ob->outputprof,pa);
	gtk_widget_set_sensitive(ob->rgbprof,ra);
	gtk_widget_set_sensitive(ob->cmykprof,ca);
	gtk_widget_set_sensitive(ob->monprof,ma);
}


static void paths_changed(GtkWidget *widget,gpointer user_data)
{
	struct prefsdialogdata *dd=(struct prefsdialogdata *)user_data;
	patheditor_get_paths(PATHEDITOR(widget),dd->gui);
	profileselector_refresh(PROFILESELECTOR(dd->rgbprof));
	profileselector_refresh(PROFILESELECTOR(dd->cmykprof));
	profileselector_refresh(PROFILESELECTOR(dd->monprof));
}


void PreferencesDialog(GtkWindow *parent,IT_GUI &gui)
{
	GtkWidget *dialog;
	GtkWidget *notebook;
	GtkWidget *profilepage;
	GtkWidget *pathspage;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *argyllpath_tg;
	GtkWidget *argyllpath_il;

	char *savedpaths=gui.GetPaths();

	struct prefsdialogdata dd;
	dd.gui=&gui;

 	dialog=gtk_dialog_new_with_buttons(_("Preferences..."),
		parent,GtkDialogFlags(0),
		GTK_STOCK_SAVE,RESPONSE_SAVE,
		GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
		GTK_STOCK_OK,GTK_RESPONSE_OK,
		NULL);

	gtk_window_set_default_size (GTK_WINDOW (dialog), 450,350);

	notebook=gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),notebook,TRUE,TRUE,0);
	gtk_widget_show(GTK_WIDGET(notebook));

	// Colour management page:

	label=gtk_label_new(_("Colour Management"));
	gtk_widget_show(label);

	profilepage=gtk_vbox_new(FALSE,0);
	gtk_widget_show(GTK_WIDGET(profilepage));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(profilepage),GTK_WIDGET(label));

	GtkWidget *table=gtk_table_new(2,4,FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(table),8);
	gtk_table_set_col_spacing(GTK_TABLE(table),0,10);
	gtk_table_set_row_spacings(GTK_TABLE(table),8);
	gtk_box_pack_start(GTK_BOX(profilepage),table,FALSE,FALSE,5);
	gtk_widget_show(table);


	dd.rgbactive=gtk_check_button_new_with_label(_("Default RGB Profile:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dd.rgbactive),gui.FindInt("DefaultRGBProfileActive"));
	g_signal_connect(G_OBJECT(dd.rgbactive),"toggled",G_CALLBACK(checkbox_changed),&dd);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.rgbactive,0,1,0,1);
	gtk_widget_show(dd.rgbactive);

	dd.rgbprof=profileselector_new(&gui,IS_TYPE_RGB,false);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.rgbprof,1,2,0,1);
	gtk_widget_show(dd.rgbprof);
	profileselector_set_filename(PROFILESELECTOR(dd.rgbprof),gui.FindString("DefaultRGBProfile"));


	dd.cmykactive=gtk_check_button_new_with_label(_("Default CMYK Profile:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dd.cmykactive),gui.FindInt("DefaultCMYKProfileActive"));
	g_signal_connect(G_OBJECT(dd.cmykactive),"toggled",G_CALLBACK(checkbox_changed),&dd);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.cmykactive,0,1,1,2);
	gtk_widget_show(dd.cmykactive);

	dd.cmykprof=profileselector_new(&gui,IS_TYPE_CMYK,false);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.cmykprof,1,2,1,2);
	gtk_widget_show(dd.cmykprof);
	profileselector_set_filename(PROFILESELECTOR(dd.cmykprof),gui.FindString("DefaultCMYKProfile"));


	dd.monactive=gtk_check_button_new_with_label(_("Monitor Profile:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dd.monactive),gui.FindInt("MonitorProfileActive"));
	g_signal_connect(G_OBJECT(dd.monactive),"toggled",G_CALLBACK(checkbox_changed),&dd);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.monactive,0,1,2,3);
	gtk_widget_show(dd.monactive);

	dd.monprof=profileselector_new(&gui,IS_TYPE_RGB,false);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.monprof,1,2,2,3);
	gtk_widget_show(dd.monprof);
	profileselector_set_filename(PROFILESELECTOR(dd.monprof),gui.FindString("MonitorProfile"));


	dd.outputactive=gtk_check_button_new_with_label(_("Default Output Profile:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dd.outputactive),gui.FindInt("PrinterProfileActive"));
	g_signal_connect(G_OBJECT(dd.outputactive),"toggled",G_CALLBACK(checkbox_changed),&dd);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.outputactive,0,1,3,4);
	gtk_widget_show(dd.outputactive);

	dd.outputprof=profileselector_new(&gui,IS_TYPE_RGB,false);
	gtk_table_attach_defaults(GTK_TABLE(table),dd.outputprof,1,2,3,4);
	gtk_widget_show(dd.outputprof);
	profileselector_set_filename(PROFILESELECTOR(dd.outputprof),gui.FindString("PrinterProfile"));


	// Paths page:

	label=gtk_label_new(_("Paths"));
	gtk_widget_show(label);

	pathspage=gtk_hbox_new(FALSE,8);
	gtk_widget_show(GTK_WIDGET(pathspage));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(pathspage),GTK_WIDGET(label));

	vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(pathspage),vbox,TRUE,TRUE,8);
	gtk_widget_show(vbox);

	label=gtk_label_new(_("ICC colour profile paths"));
	gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
	gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,8);
	gtk_widget_show(label);

	dd.patheditor = patheditor_new(&gui);
	g_signal_connect(dd.patheditor,"changed",G_CALLBACK(paths_changed),&dd);
	gtk_box_pack_start(GTK_BOX(vbox),dd.patheditor,TRUE,TRUE,0);
	gtk_widget_show(dd.patheditor);


	// Path to Argyll commands

	table=gtk_table_new(2,2,FALSE);
	gtk_table_set_col_spacing(GTK_TABLE(table),0,10);
	gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,16);
	gtk_widget_show(table);

	GtkAttachOptions gao = (GtkAttachOptions)(GTK_EXPAND|GTK_FILL);	
	
	label=gtk_label_new(_("Path to Argyll's 'tiffgamut' command"));
	gtk_table_attach(GTK_TABLE(table),label,0,1,0,1,GTK_SHRINK,gao,0,0);
	gtk_widget_show(label);

	argyllpath_tg=gtk_file_chooser_button_new(_("Path to Argyll utilities (TIFFGamut and ICCLink)"),GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_table_attach(GTK_TABLE(table),argyllpath_tg,1,2,0,1,gao,gao,0,0);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(argyllpath_tg),gui.settings.FindString("ArgyllPath_TIFFGamut"));
	gtk_widget_show(argyllpath_tg);

	label=gtk_label_new(_("Path to Argyll's 'collink' command"));
	gtk_table_attach(GTK_TABLE(table),label,0,1,1,2,GTK_SHRINK,gao,0,0);
	gtk_widget_show(label);

	argyllpath_il=gtk_file_chooser_button_new(_("Path to Argyll's collink command"),GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_table_attach(GTK_TABLE(table),argyllpath_il,1,2,1,2,gao,gao,0,0);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(argyllpath_il),gui.settings.FindString("ArgyllPath_ICCLink"));
	gtk_widget_show(argyllpath_il);

	gtk_widget_show(dialog);
	
	bool done=false;
	while(!done)
	{
		gint result=gtk_dialog_run(GTK_DIALOG(dialog));
		switch(result)
		{
			case GTK_RESPONSE_CANCEL:
				gui.ClearPaths();
				gui.AddPath(savedpaths);
				done=true;
				break;
			case GTK_RESPONSE_OK:
			case RESPONSE_SAVE:
				done=true;
				gui.SetString("DefaultRGBProfile",profileselector_get_filename(PROFILESELECTOR(dd.rgbprof)));
				gui.SetString("DefaultCMYKProfile",profileselector_get_filename(PROFILESELECTOR(dd.cmykprof)));
				gui.SetString("MonitorProfile",profileselector_get_filename(PROFILESELECTOR(dd.monprof)));
				gui.SetString("PrinterProfile",profileselector_get_filename(PROFILESELECTOR(dd.outputprof)));
				gui.SetInt("DefaultRGBProfileActive",gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dd.rgbactive)));
				gui.SetInt("DefaultCMYKProfileActive",gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dd.cmykactive)));
				gui.SetInt("MonitorProfileActive",gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dd.monactive)));
				gui.SetInt("PrinterProfileActive",gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dd.outputactive)));
				char *tmppaths=gui.GetPaths();
				gui.SetString("ProfilePath",tmppaths);
				free(tmppaths);
				gui.settings.SetString("ArgyllPath_TIFFGamut",gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(argyllpath_tg)));
				gui.settings.SetString("ArgyllPath_ICCLink",gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(argyllpath_il)));
				if(result==RESPONSE_SAVE)
				{
					gui.SaveGeometry();
					gui.SaveConfigFile();
				}
				break;
		}
	}
	gtk_widget_destroy(dialog);

	if(savedpaths)
		free(savedpaths);
}


static struct SimpleComboOption viewingconds[]=  	// Null Terminated.
{
	{"pp",N_("Practical Reflection Print")},
	{"pe",N_("Print evaluation environment")},
	{"mt",N_("Monitor in typical work environment")},
	{"mb",N_("Monitor in bright work environment")},
	{"md",N_("Monitor in darkened work environment")},
	{"jm",N_("Projector in dim environment")},
	{"jd",N_("Projector in dark environment")},
	{"pcd",N_("Photo CD - original scene outdoors")},
	{"ob",N_("Original scene - Bright Outdoors")},
	{"cx",N_("Cut Sheet Transparencies on a viewing box")},
	{NULL,NULL}
};


static struct SimpleComboOption gamutmodes[]=  	// Null Terminated.
{
	{"-ql -g ",N_("Fastest, least accurate")},
	{"-ql -G ",N_("Faster, less accurate")},
	{"-qm -g",N_("Slower, more accurate")},
	{"-qm -G",N_("Slowest, most accurate")},
	{NULL,NULL}
};



static void blurcheck_changed(GtkWidget *widget,gpointer user_data)
{
	GtkWidget *wid=GTK_WIDGET(user_data);
	bool active=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	gtk_widget_set_sensitive(wid,active);
}


static gchar *format_percentage_reversed(GtkScale *scale,gdouble value)
{
	return g_strdup_printf("%0.1f%%",100-value);
}

static gchar *format_percentage(GtkScale *scale,gdouble value)
{
	return g_strdup_printf("%0.1f%%",value);
}


bool CreateDevicelinkDialog(GtkWindow *parent,IT_GUI &gui)
{
	GtkWidget *dialog;
	GtkWidget *label;

 	dialog=gtk_dialog_new_with_buttons(_("Set devicelink creation options..."),
		parent,GtkDialogFlags(0),
		GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
		GTK_STOCK_OK,GTK_RESPONSE_OK,
		NULL);

	int oldblurstate=gui.settings.FindInt("Blur");
	float oldblurradius=gui.settings.FindFloat("BlurRadius");
	int olddsstate=gui.settings.FindInt("Downsample");
	float olddspercent=gui.settings.FindFloat("DownsamplePercent");
	int oldfilter=gui.settings.FindInt("GamutReduce");
	int oldfilterpercent=gui.settings.FindInt("GamutReducePercent");

	gtk_window_set_default_size (GTK_WINDOW (dialog), 450,350);

	GtkWidget *table=gtk_table_new(2,6,FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table),4);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),table,FALSE,FALSE,8);
	gtk_widget_show(table);

	GtkAttachOptions gao = (GtkAttachOptions)(GTK_EXPAND|GTK_FILL);	

	GtkWidget *blurradius=gtk_hscale_new_with_range(1,20,.25);
	gtk_widget_set_sensitive(GTK_WIDGET(blurradius),oldblurstate);
	gtk_range_set_value(GTK_RANGE(blurradius),oldblurradius);
	gtk_table_attach(GTK_TABLE(table),blurradius,1,2,0,1,gao,gao,5,0);
	gtk_widget_show(blurradius);

	GtkWidget *blurenabled=gtk_check_button_new_with_label(_("Gaussian Blur - Radius:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(blurenabled),oldblurstate);
	g_signal_connect(G_OBJECT(blurenabled),"toggled",G_CALLBACK(blurcheck_changed),blurradius);
	gtk_table_attach(GTK_TABLE(table),blurenabled,0,1,0,1,GTK_SHRINK,gao,5,0);
	gtk_widget_show(blurenabled);


	GtkWidget *dspercent=gtk_hscale_new_with_range(1,99,1);
	gtk_widget_set_sensitive(GTK_WIDGET(dspercent),olddsstate);
	g_signal_connect(G_OBJECT(dspercent),"format-value",G_CALLBACK(format_percentage_reversed),NULL);
	gtk_range_set_value(GTK_RANGE(dspercent),100-olddspercent);
	gtk_table_attach(GTK_TABLE(table),dspercent,1,2,1,2,gao,gao,5,0);
	gtk_widget_show(dspercent);

	GtkWidget *dsenabled=gtk_check_button_new_with_label(_("Downsample to:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dsenabled),olddsstate);
	g_signal_connect(G_OBJECT(dsenabled),"toggled",G_CALLBACK(blurcheck_changed),dspercent);
	gtk_table_attach(GTK_TABLE(table),dsenabled,0,1,1,2,GTK_SHRINK,gao,5,0);
	gtk_widget_show(dsenabled);


	GtkWidget *filterpercent=gtk_hscale_new_with_range(1,99,1);
	gtk_widget_set_sensitive(GTK_WIDGET(filterpercent),oldfilter);
	g_signal_connect(G_OBJECT(filterpercent),"format-value",G_CALLBACK(format_percentage),NULL);
	gtk_range_set_value(GTK_RANGE(filterpercent),100-oldfilterpercent);
	gtk_table_attach(GTK_TABLE(table),filterpercent,1,2,2,3,gao,gao,5,0);
	gtk_widget_show(filterpercent);

	GtkWidget *filterenabled=gtk_check_button_new_with_label(_("Popularity filter - remove:"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(filterenabled),oldfilter);
	g_signal_connect(G_OBJECT(filterenabled),"toggled",G_CALLBACK(blurcheck_changed),filterpercent);
	gtk_table_attach(GTK_TABLE(table),filterenabled,0,1,2,3,GTK_SHRINK,gao,5,0);
	gtk_widget_show(filterenabled);


	label=gtk_label_new(_("Input viewing conditions:"));
	gtk_table_attach(GTK_TABLE(table),label,0,1,3,4,GTK_SHRINK,gao,5,0);
	gtk_widget_show(label);
	
	GtkWidget *sourcevc=simplecombo_new(viewingconds);
	simplecombo_set(SIMPLECOMBO(sourcevc),gui.settings.FindString("SourceVC"));
	gtk_table_attach(GTK_TABLE(table),sourcevc,1,2,3,4,gao,gao,5,0);
	gtk_widget_show(sourcevc);

	label=gtk_label_new(_("Output viewing conditions:"));
	gtk_table_attach(GTK_TABLE(table),label,0,1,4,5,GTK_SHRINK,gao,5,0);
	gtk_widget_show(label);
	
	GtkWidget *destvc=simplecombo_new(viewingconds);
	simplecombo_set(SIMPLECOMBO(destvc),gui.settings.FindString("DestVC"));
	gtk_table_attach(GTK_TABLE(table),destvc,1,2,4,5,gao,gao,5,0);
	gtk_widget_show(destvc);

	label=gtk_label_new(_("Speed / Accuracy:"));
	gtk_table_attach(GTK_TABLE(table),label,0,1,5,6,GTK_SHRINK,gao,5,0);
	gtk_widget_show(label);
	
	GtkWidget *gamutmode=simplecombo_new(gamutmodes);
	simplecombo_set(SIMPLECOMBO(gamutmode),gui.settings.FindString("GamutMode"));
	gtk_table_attach(GTK_TABLE(table),gamutmode,1,2,5,6,gao,gao,5,0);
	gtk_widget_show(gamutmode);

	gtk_widget_show(dialog);
	
	bool rc=false;
	gint result=gtk_dialog_run(GTK_DIALOG(dialog));
	switch(result)
	{
		case GTK_RESPONSE_CANCEL:
			rc=false;
			break;
		case GTK_RESPONSE_OK:
			int newblurstate=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(blurenabled));
			float newblurradius=gtk_range_get_value(GTK_RANGE(blurradius));
			if(newblurstate!=oldblurstate || newblurradius!=oldblurradius)
			{
				gui.settings.SetInt("Blur",newblurstate);
				gui.settings.SetFloat("BlurRadius",newblurradius);
				gui.PreprocessChanged();
			}

			int newdsstate=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dsenabled));
			float newdspercent=100-gtk_range_get_value(GTK_RANGE(dspercent));
			if(newdsstate!=olddsstate || newdspercent!=olddspercent)
			{
				gui.settings.SetInt("Downsample",newdsstate);
				gui.settings.SetFloat("DownsamplePercent",newdspercent);
				gui.PreprocessChanged();
			}

			int newfilterstate=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filterenabled));
			float newfilterpercent=100-gtk_range_get_value(GTK_RANGE(filterpercent));
			if(newfilterstate!=oldfilter || newfilterpercent!=oldfilterpercent)
			{
				gui.settings.SetInt("GamutReduce",newfilterstate);
				gui.settings.SetInt("GamutReducePercent",newfilterpercent);
				gui.PreprocessChanged();
			}

			const char *prevvc=simplecombo_get(SIMPLECOMBO(sourcevc));
			if(strcmp(gui.settings.FindString("SourceVC"),prevvc)!=0)
				gui.GamutChanged();
			gui.settings.SetString("SourceVC",simplecombo_get(SIMPLECOMBO(sourcevc)));
			gui.settings.SetString("DestVC",simplecombo_get(SIMPLECOMBO(destvc)));
			gui.settings.SetString("GamutMode",simplecombo_get(SIMPLECOMBO(gamutmode)));
			rc=true;
			break;
	}
	gtk_widget_destroy(dialog);
	return(rc);
}
