#include <iostream>

#include <gtk/gtkstock.h>
#include <gtk/gtkmain.h>

#include "menu_file.h"
#include "it_gui.h"
#include "support/generaldialogs.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)

using namespace std;


static void file_open(GtkAction *action,gpointer *ob)
{
	IT_GUI *gui=(IT_GUI *)ob;
	char *filename=File_Dialog(_("Open preset..."),"",gui->window,true);

	if(filename)
	{
		gui->Load(filename);
		g_free(filename);
	}
}


static void file_save_tiff(GtkAction *action,gpointer *ob)
{
	IT_GUI *gui=(IT_GUI *)ob;
	gui->SaveTIFF();
}


static void file_save_jpeg(GtkAction *action,gpointer *ob)
{
	IT_GUI *gui=(IT_GUI *)ob;
	gui->SaveJPEG();
}


static void file_save_devicelink(GtkAction *action,gpointer *ob)
{
	IT_GUI *gui=(IT_GUI *)ob;
	gui->SaveDevicelink();
}


static void file_quit(GtkAction *action,gpointer ob)
{
	gtk_main_quit();
}


static GtkActionEntry filemenu_entries[] = {
  { "FileMenu", NULL, N_("_File") },

  { "Open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O", N_("Open an image"), G_CALLBACK(file_open) },
  { "SaveTIFF", GTK_STOCK_SAVE, N_("_Save TIFF..."), "<control>S", N_("Save targetted image in TIFF format"), G_CALLBACK(file_save_tiff) },
  { "SaveJPEG", NULL, N_("Save _JPEG..."), "<control>J", N_("Save targetted image in JPEG format"), G_CALLBACK(file_save_jpeg) },
  { "SaveDeviceLink", NULL, N_("Save _Devicelink Profile..."), "<control>D", N_("Save Devicelink profile"), G_CALLBACK(file_save_devicelink) },
  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Exit PhotoPrint"), G_CALLBACK(file_quit)},
};

static const char *filemenu_ui_description =
"<ui>"
"  <menubar name='MainMenu'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Open'/>"
"      <menuitem action='SaveTIFF'/>"
"      <menuitem action='SaveJPEG'/>"
"      <separator/>"
"      <menuitem action='SaveDeviceLink'/>"
"      <separator/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"  </menubar>"
"</ui>";


void BuildFileMenu(void *userdata,GtkUIManager *ui_manager)
{
	GError *error=NULL;
	GtkActionGroup *action_group;
	action_group = gtk_action_group_new ("FileMenuActions");
	gtk_action_group_set_translation_domain(action_group,PACKAGE);
	gtk_action_group_add_actions (action_group, filemenu_entries, G_N_ELEMENTS (filemenu_entries), userdata);
	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
	
	if (!gtk_ui_manager_add_ui_from_string (ui_manager, filemenu_ui_description, -1, &error))
		throw error->message;
}
